<!-----------------------------------------------------------------------
Author: Luis Majano
Date: JSeptember 27, 2005
Description: This is the framework's logger object. It performs all necessary
			 loggin procedures.

Modifications:
09/25/2005 - Created the template.
10/13/2005 - Updated the reqCollection to the request scope.
12/18/2005 - Using mailserverSettings from config.xml now, fixed structnew for default to logError
12/20/2005 - Bug on spaces for log files.
02/16/2006 - Fixes for subjects
06/28/2006 - Updates for coldbox
----------------------------------------------------------------------->
<cfcomponent name="logger"
			 hint="This is the frameworks logger object. It is used for all logging procedures."
			 extends="plugin">

	<!--- ************************************************************* --->
	<cffunction name="init" access="public" returntype="any" hint="Constructor" output="false">
		<cfargument name="controller" required="yes" hint="The reference to the framework controller">
		<cfset super.Init(arguments.controller) />
		<cfreturn this>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="tracer" access="Public" hint="Log a trace message" output="false" returntype="void">
		<!--- ************************************************************* --->
		<cfargument name="message"    hint="Message to Send" type="string" required="Yes" >
		<cfargument name="ExtraInfo"  hint="Extra Information to dump on the trace" required="No" default="" type="any">
		<!--- ************************************************************* --->
		<cfscript>
		var tracerEntry = StructNew();
		if ( controller.getDebugMode() ){
			if ( not structkeyExists(request.reqCollection, "tracerStack") )
				request.reqCollection.tracerStack = ArrayNew(1);
			//Insert Message & Info
			StructInsert(tracerEntry,"message", arguments.message);
			if ( not isSimpleValue(arguments.ExtraInfo) )
				StructInsert(tracerEntry,"ExtraInfo", duplicate(arguments.ExtraInfo));
			else
				StructInsert(tracerEntry,"ExtraInfo", arguments.ExtraInfo);
		    ArrayAppend(request.reqCollection.tracerStack,tracerEntry);
		}
		</cfscript>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="logErrorWithBean" access="public" hint="Log an error into the framework using a coldbox exceptionBean" output="false" returntype="void">
		<!--- ************************************************************* --->
		<cfargument name="ExceptionBean" 	type="any" 	required="yes">
		<!--- ************************************************************* --->
		<!--- Initialize variables --->
		<cfset var Exception = arguments.ExceptionBean>
		<cfset var BugReport = "">
		<cfset var logSubject = "">
		<cfset var errorText = "">
		<cfset var arrayTagContext = "">

		<!--- Coldfusion Loggin Enabled? --->
		<cfif getSetting("ColdfusionLogging")>
			<cfset errorText = "#getSetting("AppName")# Error. Extra Message: #Exception.getExtraMessage()#.">
			<cfif isStruct(Exception.getExceptionStruct())>
				<cfif Exception.getDetail() neq  "">
					<cfset errorText = errorText & " Details: #Exception.getDetail()# ">
				</cfif>
				<cfif Exception.getMessage() neq "">
					<cfset errorText = errorText & " Message: #Exception.getMessage()#">
				</cfif>
				<cfif Exception.getStackTrace() neq "">
					<cfset errorText = errorText & chr(13) & chr(13) & "#Exception.getStackTrace()#">
				</cfif>
			</cfif>
			<!--- Coldfusion Log Entry --->
			<cflog type="error"
				   text="#errorText#"
				   file="#replace(getSetting("AppName")," ","","all")#_BugTrackingErrors">
		</cfif>

		<!--- Save the Bug Report --->
		<cfsavecontent variable="BugReport"><cfinclude template="../includes/BugReport.cfm"></cfsavecontent>

		<!--- Check if Bug Reports are Enabled, then send Emails. --->
		<cfif getSetting("EnableBugReports") and getSetting("BugEmails") neq "">
			<!--- Setup The Subject --->
			<cfset logSubject = "#getSetting("Codename",1)# Bug Report: #getSetting("Environment")# - #getSetting("appname")#">
			<!--- Check for Custom Mail Settings or use CFMX Administrator Settings --->
			<cfif getSetting("MailServer") neq "">
				<!--- Mail New Bug --->
				<cfmail to="#getSetting("BugEmails")#"
						from="#getSetting("OwnerEmail")#"
						subject="#logSubject#"
						type="html"
						server="#getSetting("MailServer")#"
						username="#getSetting("MailUsername")#"
						password="#getSetting("MailPassword")#">#BugReport#</cfmail>
			<cfelse>
				<!--- Mail New Bug --->
				<cfmail to="#getSetting("BugEmails")#"
						from="#getSetting("OwnerEmail")#"
						subject="#logSubject#"
						type="html"
						username="#getSetting("MailUsername")#"
						password="#getSetting("MailPassword")#">#BugReport#</cfmail>
			</cfif>
		</cfif>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="logError" access="public" hint="Log an error into the framework using arguments. Facade to logErrorWithBean." output="false" returntype="void">
		<!--- ************************************************************* --->
		<cfargument name="Message" 			type="string" 	required="yes">
		<cfargument name="ExceptionStruct" 	type="any"    	required="no" default="#StructNew()#" hint="The CF cfcatch structure.">
		<cfargument name="ExtraInfo"  		type="any"    	required="no" default="">
		<!--- ************************************************************* --->
		<cfset logErrorWithBean(getPlugin("beanFactory").create("beans.exception").init(arguments.ExceptionStruct,arguments.message,arguments.ExtraInfo))>
	</cffunction>
	<!--- ************************************************************* --->

</cfcomponent>