<!-----------------------------------------------------------------------
Author 	 :	Luis Majano
Date     :	September 23, 2005
Description :
	This is ColdBox's renderer plugin. In high traffic sites if using
	cfsavecontent for the generated view or layout, java.lang.illegalStateExceptions
	will be produced sporadically.  This has been logged with Adobe as a bug
	on the cfmodule tag.  Thus, it uses a output=true and a simple
	cfinclude.

Modification History:
10/13/2005 - Upgraded the reqCollection to the request scope.
12/23/2005 - Eliminate the dump of application structures.
01/03/2006 - Added var results = "" to init.
01/20/2006 - Removed Dumpvar, to debug template.
06/28/2006 - Updated for Coldbox.
----------------------------------------------------------------------->
<cfcomponent name="renderer" hint="This plugin renders layouts, views, framework includes, etc." extends="plugin">

	<!--- ************************************************************* --->
	<cffunction name="init" access="public" returntype="any" output="false">
		<cfargument name="controller" required="yes" hint="The reference to the framework controller">
			<cfset super.Init(arguments.controller) />
		<cfreturn this>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="renderView"	access="Public" hint="Renders the current view." output="false" returntype="Any">
		<cfset var RenderedView = "">
		<cfset var AppDir = "">
		<cfmodule template="../includes/timer.cfm" timertag="Rendering View [#getvalue('currentView','')#]">
			<cfif CompareNocase(getSetting("AppName"),getSetting("DashboardName",1)) eq 0>
				<cfset AppDir = "../admin">
			<cfelse>
				<cfset AppDir = "/#getSetting("AppCFMXMapping")#">
			</cfif>
			<cfif not valueExists("currentView")>
				<cfthrow type="Framework.ViewNotSetException" message="Framework.renderView: The currentview variable has not been set, therefore there is no view to render.">
			</cfif>
			<cfif not valueExists("currentLayout")>
				<!--- UDF Library Call --->
				<cfset includeUDF()>
			</cfif>
			<cfsavecontent variable="RenderedView"><cfinclude template="#AppDir#/views/#getValue("currentView")#.cfm"></cfsavecontent>
		</cfmodule>
		<cfreturn RenderedView>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="renderLayout" access="Public" hint="Renders the current layout." output="false" returntype="Any">
		<cfset var RederedLayout = "">
		<cfset var AppDir = "">
		<cfmodule template="../includes/timer.cfm" timertag="Rendering Layout [#getvalue('currentLayout','')#]">
			<cfif CompareNocase(getSetting("AppName"),getSetting("DashboardName",1)) eq 0>
				<cfset AppDir = "../admin">
			<cfelse>
				<cfset AppDir = "/#getSetting("AppCFMXMapping")#">
			</cfif>
			<!--- Render With No Layout --->
			<cfif not valueExists("currentLayout")>
				<cfset RederedLayout = renderView()>
			<cfelse>
				<!--- UDF Library Call --->
				<cfset includeUDF()>
				<cfsavecontent variable="RederedLayout"><cfinclude template="#AppDir#/layouts/#getValue("currentLayout")#"></cfsavecontent>
			</cfif>

		</cfmodule>
		<cfreturn RederedLayout>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="renderDebugLog" access="public" hint="Return the debug log." output="false" returntype="Any">
		<cfset var RenderedDebugging = "">
		<cfsavecontent variable="RederedDebugging"><cfinclude template="../includes/debug.cfm"></cfsavecontent>
		<cfreturn RederedDebugging>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="renderBugReport" access="public" hint="Render a Bug Report." output="false" returntype="Any">
		<cfargument name="ExceptionBean" type="any" required="true">
		<cfset var BugReport = "">
		<cfset var Exception = arguments.ExceptionBean>
		<!--- test for custom bug report --->
		<cfif Exception.getErrortype() eq "application" and getSetting("CustomErrorTemplate") neq "">
			<cftry>
				<!--- Place exception in the requset Collection --->
				<Cfset setvalue("ExceptionBean",Exception)>
				<!--- Save the Custom Report --->
				<cfsavecontent variable="BugReport"><cfinclude template="/#getSetting("AppCFMXMapping")#/#getSetting("CustomErrorTemplate")#"></cfsavecontent>
				<cfcatch type="any">
					<cfset Exception = getPlugin("settings").ExceptionHandler(cfcatch,"Application","Error creating custom error template.")>
					<!--- Save the Bug Report --->
					<cfsavecontent variable="BugReport"><cfinclude template="../includes/BugReport.cfm"></cfsavecontent>
				</cfcatch>
			</cftry>
		<cfelse>
			<!--- Save the Bug Report --->
			<cfsavecontent variable="BugReport"><cfinclude template="../includes/BugReport.cfm"></cfsavecontent>
		</cfif>
		<cfreturn BugReport>
	</cffunction>
	<!--- ************************************************************* --->

	<!--- ************************************************************* --->
	<cffunction name="includeUDF" access="public" hint="Includes the UDF Library if found and exists. Called only by the framework." output="false" returntype="void">
		<!--- check if UDFLibraryFile is defined  --->
		<cfif getSetting("UDFLibraryFile") neq "">
			<!--- Check if file exists on app's includes --->
			<cfif fileExists("#getSetting("ApplicationPath",1)#/#getSetting("UDFLibraryFile")#")>
				<cfinclude template="/#getSetting("AppCFMXMapping")#/#getSetting("UDFLibraryFile")#">
			<cfelseif fileExists(ExpandPath("#getSetting("UDFLibraryFile")#"))>
				<cfinclude template="#getSetting("UDFLibraryFile")#">
			<cfelse>
				<cfthrow type="Framework.UDFLibraryNotFoundException" message="Error loading UDFLibraryFile.  The file declared in the config.xml: #getSetting("UDFLibraryFile")# was not found in your application's include directory or in the following location: #ExpandPath(getSetting("UDFLibraryFile"))#. Please make sure you verify the file's location.">
			</cfif>
		</cfif>
	</cffunction>
	<!--- ************************************************************* --->

</cfcomponent>